/*
 * Digital AC Dimmer
 *
 * Copyright (c) 2005-2006 Chris Fallin <chris@cfallin.org>
 * Placed under the modified BSD license
 *
 * dim.h: interrupt-driven AC dimmer module
 */

#ifndef _DIMMER_H_
#define _DIMMER_H_

void dim_initialize();
void dim_set_level(char l);

#endif
