/*
 * Digital AC Dimmer
 *
 * Copyright (c) 2005-2006 Chris Fallin <chris@cfallin.org>
 * Placed under the modified BSD license
 *
 * dmx.h: DMX-512 receiver module
 */

#ifndef _DMX_H_
#define _DMX_H_

void dmx_initialize();

#endif
