/*
 * Digital AC Dimmer
 *
 * Copyright (c) 2005-2006 Chris Fallin <chris@cfallin.org>
 * Placed under the modified BSD license
 *
 * main.c: main code for dimmer module
 */

#include "dim.h"
#include "dmx.h"
#include <avr/io.h>
#include <avr/interrupt.h>

extern int lvl;

int main()
{
  dmx_initialize();
  dim_initialize();
  sei(); // enable interrupts

  while(1); // the interrupts do the rest
}
